import logging
import os

from rich.console import Console
from rich.table import Table
from rich.live import Live

from .game import Pydle
from ._stats import Stats
from . import app_paths

logging.basicConfig(filename=app_paths.log_file_path, level=logging.INFO)
_logger = logging.getLogger(__name__)


def clear():
    if os.name == "nt":
        _ = os.system("cls")
    else:
        _ = os.system("clear")


def _print_graph(attempts):
    import termplotlib as tpl

    attempts = dict(sorted(attempts.items()))

    fig = tpl.figure()
    fig.barh(list(attempts.values()), list(attempts.keys()), force_ascii=True)
    fig.show()


def run():
    stats = Stats()
    game = Pydle()
    console = Console()
    table = Table(title="Guesses", box=None)
    console.print(table)

    found: bool = False
    MAX_ATTEMPTS: int = 6
    retries: int = 6

    while retries and not found:
        game.get_user_guess(remaining=retries)
        found, result = game.process_user_guess()
        clear()
        with Live(table):
            guess_row = [
                f'[black on {res["color"]}] {res["letter"]} [/black on {res["color"]}]'
                for res in result
            ]
            table.add_row(*guess_row)
            table.add_row("")
            retries -= 1

    game_stats = {
        "attempts": MAX_ATTEMPTS - retries,
        "found": found,
    }

    stats.update(game_stats)

    if found:
        console.print(
            f"\n :thumbs_up: Wow, you aced it in {MAX_ATTEMPTS - retries} guesses!!\n"
        )
    else:
        console.print(
            f"\n\n☹️  [bold red]Correct Word is {game._word_choice.upper()} [/bold red]"
        )

    total_stats = stats.parse_stats()
    console.print(
        f'You have won {total_stats["wins"]} times out of {total_stats["games"]}!\n'
        f'That\'s {int(total_stats["wins"] / total_stats["games"] * 100)}% !'
    )
    try:
        console.print("[bold]Player Statistics[/bold]")
        total_attempts = total_stats["attempts"]
        for i in range(1, MAX_ATTEMPTS+1):
            if i not in total_attempts:
                total_attempts[i] = 0
        _print_graph(total_attempts)
    except:
        return


if __name__ == "__main__":
    run()



#!/usr/bin/env python3
# -*- coding=utf-8 -*-
"""
Pydle game player statistics
"""

import json
from collections import Counter
from pathlib import Path
from typing import List, Optional

from . import app_paths


StatData = List[dict]


class Stats:
    def __init__(self):
        self._datastore = Path(app_paths.app_data_path) / "player_stats.jsonl"
        self._stats: Optional[StatData] = None

    def update(self, game_stats: dict) -> None:
        """Update player stats with new game"""
        with self._datastore.open("a+") as fileobj:
            print(json.dumps(game_stats), file=fileobj)

    def get_stats(self) -> StatData:
        """Read player stats file"""
        if not self._stats:
            _stats = []
            with self._datastore.open("r") as fileobj:
                for line in fileobj.readlines():
                    _stats.append(json.loads(line))
            self._stats = _stats
        return self._stats

    def parse_stats(self, data: Optional[StatData] = None) -> dict:
        """Parse stats file for total info"""
        data = data or self.get_stats()
        return {
            "games": len(data),
            "wins": sum(1 for row in data if row.get("found", False) == True),
            "attempts": Counter([row.get("attempts") for row in data if row.get("found")]),
        }


if __name__ == "__main__":
    print("This is not the py you seek")



import json
import logging
import random
from collections import Counter
from pathlib import Path
from typing import List, Optional, Tuple, Union

from rich.prompt import Prompt
from rich import print


_logger = logging.getLogger(__name__)

FilePath = Union[str, Path]


class Pydle:
    def __init__(self, word_file: Optional[FilePath] = None):
        _default_word_file = Path(__file__).parent.absolute() / "data" / "words.json"
        # Give the option for a user to supply a json list of words
        # Otherwise use our default list
        word_file = Path(word_file) if word_file else _default_word_file
        _logger.info("Word file being used is %s", str(word_file.absolute()))
        if not word_file.exists():
            _logger.error("Word file %s does not exist", str(word_file))
            raise FileNotFoundError(str(word_file))

        self._word_options: List[str] = self.get_word_options(word_file)
        self._word_choice: str = self._select_word()
        self._word_length: int = 5
        self._guess: str = ""

    @staticmethod
    def get_word_options(word_file: Path) -> List[str]:
        with word_file.open("rt") as f:
            words = json.load(f)
        return words

    def _select_word(self) -> str:
        return random.choice(self._word_options)

    def get_user_guess(self, remaining: int = None) -> str:
        self._guess = Prompt.ask(
            f"\n\n[gray]Guess{f' ({remaining} remaining)' if remaining else ''}[/gray]"
        ).strip()
        if len(self._guess) != self._word_length:
            print(f"[red]ERROR: Guess must be a {self._word_length} letter word[/red]")
            self.get_user_guess(remaining=remaining)
        if self._guess not in self._word_options:
            print(f"[red]ERROR: Guess is not actually a word!.[/red]")
            self.get_user_guess(remaining=remaining)
        return self._guess

    def process_user_guess(self) -> Tuple[bool, List[dict]]:
        answer_letters = list(self._word_choice)
        answer_count = Counter(answer_letters)
        guess_letters = list(self._guess)

        guess_info = []
        for guess, answer in zip(guess_letters, answer_letters):
            letter_info = {"letter": guess, "color": "grey84"}
            if guess == answer:
                letter_info["color"] = "spring_green2"
                answer_count[guess] -= 1
            guess_info.append(letter_info)

        for i, square in enumerate(guess_info):
            letter = square["letter"]
            if letter in answer_letters:
                if answer_count[letter] > 0 and square["color"] == "grey84":
                    guess_info[i]["color"] = "yellow3"
                    answer_count[letter] -= 1

        return self._is_correct(), guess_info

    def _is_correct(self):
        return self._word_choice == self._guess


if __name__ == "__main__":
    print("This is not the py you seek")



#!/usr/bin/env python
"""
The only reason for the existence of this setup.py shim is to support editable
installs while developing and testing this package.
"""
import setuptools

if __name__ == "__main__":
    setuptools.setup()
